@@ -26,6 +26,7 @@
 # JSSAk = 28
 # JSSAkCL = 29
 
+# secr = 5
 # CJSsecr = 6
 # JSSAsecrf = 7
 # JSSAsecrD = 8
@@ -38,9 +39,13 @@
 # JSSAsecrg = 24
 # JSSAsecrgCL = 25
 
+# secrCL = 30
+# secrD = 31
+
 #---------------------------------------------------------
 
-open.loglikfn <- function (beta, dig = 3, betaw = 8, cluster = NULL, oneeval = FALSE, data)
+## open.loglikfn <- function (beta, dig = 3, betaw = 8, cluster = NULL, oneeval = FALSE, data)
+open.loglikfn <- function (beta, dig = 3, betaw = 8, oneeval = FALSE, data)
 
     # Return the negative log likelihood
     # Transformed parameter values are passed in the vector 'beta'
@@ -82,7 +87,7 @@ open.loglikfn <- function (beta, dig = 3, betaw = 8, cluster = NULL, oneeval = F
     type <- typecode(data$type)
     if (type<0) stop ("Invalid likelihood type")
     if (type %in% 28:29 & any(data$intervals !=1)) stop ("kappa parameterisation available only if all intervals = 1")
-    
+
     PIA <- data$design$PIA
     PIAJ <- data$design$PIAJ
 
@@ -110,14 +115,13 @@ open.loglikfn <- function (beta, dig = 3, betaw = 8, cluster = NULL, oneeval = F
                 as.integer(data$nc),         ## needed for nrows of PIAJ
                 as.integer(data$J),
                 as.integer(data$details$nmix),
-                as.double(realparval),
-                as.integer(nrow(realparval)),       # number of rows in lookup table
+                as.matrix(realparval),
                 as.integer(PIAJ),             # index of nc,S,mix to rows
                 as.double(data$intervals))                # number of interval == J-1
         }
     }
     else if (data$details$R & (type %in% c(28,29))) {
-        comp <- kappaloglik (type, realparval,  PIA, PIAJ, data) 
+        comp <- kappaloglik (type, realparval,  PIA, PIAJ, data)
     }
     else {
         onehistory <- function (n, pmix) {
@@ -155,8 +159,7 @@ open.loglikfn <- function (beta, dig = 3, betaw = 8, cluster = NULL, oneeval = F
                             as.integer(data$capthist[n,]),
                             as.integer(data$fi[n]),
                             as.integer(data$li[n]),              # may be negative if censored 2018-01-17
-                            as.double (realparval),
-                            as.integer(nrow(realparval)),
+                            as.matrix (realparval),
                             as.integer(PIA[n,,,]),
                             as.integer(PIAJ[n,,]),
                             as.double (data$intervals),
@@ -170,21 +173,48 @@ open.loglikfn <- function (beta, dig = 3, betaw = 8, cluster = NULL, oneeval = F
                 else
                     freq[n] * log(sump)
         }
+        allhistparallel <- function () {
+            sump <- numeric(data$nc)
+            for (x in 1:nrow(pmix)) {
+                temp <-  allhistparallelcpp(
+                    as.integer(x-1),
+                    as.integer(type),
+                    as.integer(data$nc),
+                    as.integer(data$details$CJSp1),
+                    as.integer(data$details$grain),
+                    as.matrix (pmix),
+                    as.double (data$intervals),
+                    as.integer(data$cumss),
+                    as.integer(data$capthist),
+                    as.integer(data$fi),
+                    as.integer(data$li),
+                    as.matrix (realparval),
+                    as.integer(PIA),
+                    as.integer(data$design$PIAJ))
+
+                sump <- sump + freq * log(temp)
+            }
+            sump  ## return vector of individual LL contributions
+        }
+
 
         comp <- numeric(5)
         pmix <- fillpmix2(data$nc, data$details$nmix, PIA, realparval)
-
         #####################################################################
         # Component 1: Probability of observed histories - all models
-        if (is.null(cluster))
-            temp <- sapply(1:data$nc, onehistory, pmix = pmix)
-        else
-            temp <- parSapply(cluster, 1:data$nc, onehistory, pmix = pmix, USE.NAMES = FALSE )
+        # if (is.null(cluster)) {
+            if (data$ncores>1)
+                temp <- allhistparallel()
+            else
+                temp <- sapply(1:data$nc, onehistory, pmix = pmix)
+        # }
+        # else {
+        #         temp <- parSapply(cluster, 1:data$nc, onehistory, pmix = pmix, USE.NAMES = FALSE )
+        # }
 
         comp[1] <- sum(temp)
         #####################################################################
         # Component 2: Probability of missed animals (all-zero histories)
-
         if (type %in% c(2:4,15:19, 21, 22, 23, 27, 28, 29)) {
             pdot <- rep(0, data$nc)
             if (data$learnedresponse)
@@ -212,10 +242,9 @@ open.loglikfn <- function (beta, dig = 3, betaw = 8, cluster = NULL, oneeval = F
                             as.integer(data$J),
                             as.integer(data$cumss),
                             as.integer(data$details$nmix),
-                            as.double(unlist(realparval0)),
-                                 as.integer(nrow(realparval0)),
-                                 as.integer(PIA0),
-                                 as.integer(PIAJ),
+                            as.matrix(realparval0),
+                            as.integer(PIA0),
+                            as.integer(PIAJ),
                             as.double(data$intervals))
                 }
                 pdot <- pdot + pmix[x] * pch1
@@ -230,24 +259,7 @@ open.loglikfn <- function (beta, dig = 3, betaw = 8, cluster = NULL, oneeval = F
                 superN <- realparval[nrow(realparval)*3+1] # Nsuper direct
             }
             else {
-                # if (data$details$R) {
-                    superN <- getN(type, ncf, data$J, data$details$nmix, pmix, realparval, PIAJ, data$intervals)
-                # }
-                # else {
-                #     temp <- getNcpp(
-                #         as.integer(type),
-                #         as.integer(data$nc),
-                #         as.integer(ncf),
-                #         as.integer(data$J),
-                #         as.integer(data$details$nmix),
-                #         as.double(pmix),
-                #         as.double(data$intervals),
-                #         as.double(realparval),
-                #         as.integer(nrow(realparval)),
-                #         as.integer(PIAJ))
-                #     
-                #     superN <- temp[1]
-                # }
+                superN <- getN(type, ncf, data$J, data$details$nmix, pmix, realparval, PIAJ, data$intervals)
             }
             meanpdot <- ncf / sum(1/rep(pdot,freq))  ## cf CLmeanesa in 'secr'
             comp[3] <- switch (data$distrib+1,
@@ -256,7 +268,7 @@ open.loglikfn <- function (beta, dig = 3, betaw = 8, cluster = NULL, oneeval = F
                                lnbinomial (ncf, superN + ncf, meanpdot),
                                NA)
         }
-        
+
     }
 
     ## optional multinomial term
@@ -297,14 +309,12 @@ open.loglikfn <- function (beta, dig = 3, betaw = 8, cluster = NULL, oneeval = F
 
 }
 
-open.secr.loglikfn <- function (beta, dig = 3, betaw = 8, cluster = NULL, oneeval = FALSE, data)
+open.secr.loglikfn <- function (beta, dig = 3, betaw = 8, oneeval = FALSE, data)
 
     # Return the negative log likelihood
     # Transformed parameter values are passed in the vector 'beta'
     # details$trace=T sends a one-line report to the screen
 
-    # an existing cluster is used
-
 {
     #--------------------------------------------------------------------
     # Fixed beta
@@ -334,44 +344,25 @@ open.secr.loglikfn <- function (beta, dig = 3, betaw = 8, cluster = NULL, oneeva
     #-----------------------------------------
     type <- typecode(data$type)
     if (type < 0) stop ("Invalid likelihood type")
-    
+
     trps <- traps(data$capthist)
     if (!is.null(data$mask)) area <- attr(data$mask,'area')
     else area <- 0
-    
-    binomN <- switch (detector(trps)[1], multi = 1, proximity = 1, count = data$binomN, -1)
-    if (binomN < 0)
+
+    binomN <- switch (detector(trps)[1], multi = -1, proximity = -1, count = data$binomN, -9)
+    if (binomN < -2)
         stop("open-population secr requires multi, proximity or count detector type")
-    
-    if (data$details$debug>1) {
-        print(type)
-        print(summary(updateCH(data$capthist)))
-        print(data$nc)
-        print(data$J)
-        print(data$k)
-        print(data$m)
-        print(data$cumss)
-        print(data$details)
-        print(summary(trps))
-        print(summary(data$mask))
-        print(area)
-        print(data$design)
-        print(realparval)
-        print(nrow(realparval))
-        print(data$intervals)
-        print(data$movemodel)
-        print(binomN)
-    }
-    
+
     if (data$details$debug>2) browser()
-    
+
     PIA <- data$design$PIA
     PIAJ <- data$design$PIAJ
     if (data$learnedresponse)
         PIA0 <- data$design0$PIA
     else
         PIA0 <- PIA
-    
+    #-----------------------------------------
+
     onehistory <- function (n, pmix, gk) {
         sump <- 0
         if (data$details$R) {
@@ -422,235 +413,281 @@ open.secr.loglikfn <- function (beta, dig = 3, betaw = 8, cluster = NULL, oneeva
                     as.integer(data$fi[n]),
                     as.integer(data$li[n]),
                     as.double (gk),                ## precomputed probability
-                    as.double (realparval),
-                    as.integer(nrow(realparval)),
+                    as.matrix (realparval),
                     as.integer(PIA[n,,,, drop = FALSE]),
                     as.integer(data$design$PIAJ[n,,]),
                     as.integer(binomN),
-                    as.double (data$usge),
+                    as.matrix (data$usge),
                     as.double (data$intervals),
                     as.integer(data$moveargsi),
+                    as.integer(-1),                  ## not multicatch
+                    as.integer(-1),                  ## not multicatch
                     as.integer(data$details$CJSp1),
                     as.integer(data$movemodel),
                     as.character(data$usermodel),
-                    as.integer(nrow(data$kernel)),
-                    as.integer(as.matrix(data$kernel)),
-                    as.integer(data$mqarray),
+                    as.matrix(data$kernel),
+                    as.matrix(data$mqarray),
                     as.double (data$cellsize))
                 sump <- sump + pmix[x,n] * temp
             }
         }
         freq[n] * log(sump)
     }
+
     onehistorymulti <- function (n, pmix, hk) {
         sump <- 0
         for (x in 1:nrow(pmix)) {
             if (data$details$R) {
                 temp <- prwisecrmulti (
                     type,
-                    1, x, 
+                    1, x,
                     data$J,
-                    data$m, 
-                    data$cumss, 
-                    data$capthist[n,,drop = FALSE], 
-                    data$fi[n], 
-                    data$li[n], 
-                    hk, 
+                    data$m,
+                    data$cumss,
+                    data$capthist[n,,drop = FALSE],
+                    data$fi[n],
+                    data$li[n],
+                    hk,
                     realparval,
                     PIA[n,,,,drop = FALSE],
                     PIAJ[n,,,drop = FALSE],
-                    binomN, 
-                    data$usge, 
-                    data$intervals,  
-                    data$moveargsi, 
-                    haztemp$h, 
-                    haztemp$hindex[n,,drop = FALSE]+1, 
-                    data$details$CJSp1, 
+                    binomN,
+                    data$usge,
+                    data$intervals,
+                    data$moveargsi,
+                    haztemp$h,
+                    haztemp$hindex[n,,drop = FALSE]+1,
+                    data$details$CJSp1,
                     data$movemodel,
-                    data$usermodel, 
-                    data$kernel, 
-                    data$mqarray, 
+                    data$usermodel,
+                    data$kernel,
+                    data$mqarray,
                     data$cellsize)
             }
             else {
-                temp <- prwisecrmulticpp(
+                hx <- matrix(haztemp$h[x,,], nrow = data$m)
+                temp <- prwisecrcpp(
                     as.integer(type),
                     as.integer(0),                       ## always just one n
                     as.integer(x-1),
                     as.integer(1),                       ## nc one at a time
                     as.integer(data$J),
-                    as.integer(data$cumss),
                     as.integer(data$k),
                     as.integer(data$m),
                     as.integer(data$details$nmix),
-                    as.integer(data$capthist[n,]),       ## 2-D CH
+                    as.integer(data$cumss),
+                    as.integer(data$capthist[n,,drop = FALSE]),       ## 2-D CH
                     as.integer(data$fi[n]),
                     as.integer(data$li[n]),
                     as.double (hk),                      ## hazard instead of probability
-                    as.double (realparval),
-                    as.integer(nrow(realparval)),
+                    as.matrix (realparval),
                     as.integer(PIA[n,,,]),
                     as.integer(PIAJ[n,,]),
                     as.integer(binomN),
-                    as.double (data$usge),
+                    as.matrix (data$usge),
                     as.double (data$intervals),
                     as.integer(data$moveargsi),
-                    as.double (haztemp$h),               ## lookup sum_k (hazard)
-                    as.integer(haztemp$hindex[n,]),     ## index to h
+                    as.matrix (hx),               ## lookup sum_k (hazard)
+                    as.matrix(haztemp$hindex[n,,drop = FALSE]),      ## index to h
                     as.integer(data$details$CJSp1),
                     as.integer(data$movemodel),
                     as.character(data$usermodel),
-                    as.integer(nrow(data$kernel)),
-                    as.integer(as.matrix(data$kernel)),
-                    as.integer(data$mqarray),
+                    as.matrix(data$kernel),
+                    as.matrix(data$mqarray),
                     as.double (data$cellsize))
             }
-            # cat('n ', n, ' LL ', temp, '\n')
             sump <- sump + pmix[x,n] * temp
         }
         freq[n] * log(sump)
+    }
+    allhistsecrparallel <- function () {
+        sump <- numeric(data$nc)
+        for (x in 1:nrow(pmix)) {
+            hx <- matrix(haztemp$h[x,,], nrow = data$m)
+            temp <-  allhistsecrparallelcpp(
+                as.integer(x-1),
+                as.integer(type),
+                as.integer(data$m),
+                as.integer(data$nc),
+                as.integer(binomN),
+                as.integer(data$details$CJSp1),
+                as.integer(data$details$grain),
+                as.matrix (pmix),
+                as.double (data$intervals),
+                as.integer(data$cumss),
+                as.matrix (data$capthist),     ## 2-D CH
+                as.integer(data$fi),
+                as.integer(data$li),
+                as.double (hk),                ## precomputed probability
+                as.matrix (realparval),
+                as.integer(PIA),
+                as.integer(data$design$PIAJ),
+                as.matrix (data$usge),
+                as.matrix (hx),                ## lookup sum_k (hazard)
+                as.matrix (haztemp$hindex),     ## index to h
+                as.integer(data$movemodel),
+                as.integer(data$moveargsi),
+                as.matrix (data$kernel),
+                as.matrix (data$mqarray),
+                as.double (data$cellsize)
+            )
+            sump <- sump + freq * log(temp)
         }
-    temp <- makegkcpp(
-        as.integer(nrow(realparval)),      # number of rows in lookup table
-        as.integer(data$k),                # detectors
-        as.integer(data$m),                # mask points
-        as.integer(data$detectfn),
-        as.integer(.openCRstuff$sigmai[type]),                # column index of sigma in realparval
-        as.double(realparval),
-        as.double(unlist(trps)),
-        as.double(unlist(data$mask)))
+        sump
+    }
+    #-----------------------------------------
+
+    ## number of threads was set in openCR.fit
+    temp <- makegkParallelcpp (as.integer(data$detectfn),
+                               as.integer(.openCRstuff$sigmai[type]),
+                               as.integer(data$details$grain),
+                               as.matrix(realparval),
+                               as.matrix(trps),
+                               as.matrix(data$mask))
     gk <- array(temp[[1]], dim=c(nrow(realparval), data$k, data$m))  # array form for R use
     hk <- array(temp[[2]], dim=c(nrow(realparval), data$k, data$m))  # array form for R use
-
-    # ch <- split.default(capthist, slice.index(capthist,1))
-    # pia <- split.default(PIA, slice.index(PIA, 1))
-    
-    comp <- numeric(5)
-    
     pmix <- fillpmix2(data$nc, data$details$nmix, PIA, realparval)
     S <- ncol(data$capthist)
-    
-    haztemp <- gethcpp(
-        as.integer(data$nc),
-        as.integer(nrow(realparval)),
-        as.integer(data$details$nmix),
-        as.integer(data$k),
-        as.integer(data$J),
-        as.integer(data$m),
-        as.integer(PIA),
-        as.integer(data$cumss),
-        as.double(data$usge),
-        as.double(hk))
-    names(haztemp) <- c('hc0','h','hindex')
-    haztemp$hindex <- matrix(haztemp$hindex, nrow = data$nc)
-    ## discard surplus h for speed in C   DISCARD INTERNALLY
-    haztemp$h <- haztemp$h[1:(data$m * data$details$nmix * (max(haztemp$hindex)+1))]
-    if (data$details$R) {
+
+    #-----------------------------------------
+
+    if (data$multi) {
+        haztemp <- gethcpp(
+            as.integer(data$nc),
+            as.integer(nrow(realparval)),
+            as.integer(data$details$nmix),
+            as.integer(data$k),
+            as.integer(data$J),
+            as.integer(data$m),
+            as.integer(PIA),
+            as.integer(data$cumss),
+            as.double(data$usge),
+            as.double(hk))
+        names(haztemp) <- c('hc0','h','hindex')
+        haztemp$hindex <- matrix(haztemp$hindex, nrow = data$nc)
+        ## discard surplus h for speed in C   DISCARD INTERNALLY
+        haztemp$h <- haztemp$h[1:(data$m * data$details$nmix * (max(haztemp$hindex)+1))]
         haztemp$h <- array(haztemp$h, dim = c(data$details$nmix, data$m, max(haztemp$hindex)+1))
     }
+    else {
+        haztemp <- list(h = array(-1, dim=c(data$details$nmix,1,1)), hindex = matrix(-1))
+    }
+    #####################################################################
+
+    ## Vector to store components of log likelihood
+    comp <- numeric(5)
+
     #####################################################################
     # Component 1: Probability of observed histories
-    
-    if (is.null(cluster)) {
+
+    if (data$ncores>1)
+        temp <- allhistsecrparallel()
+    else {
         if (data$multi) {
-            ## temp <- allhistoriesmulti(pmix, hk)  ## little speed gain, or slower
             temp <- sapply(1:data$nc, onehistorymulti, pmix = pmix, hk = hk, USE.NAMES = FALSE )
         }
         else {
-            ## temp <- allhistories(pmix, gk)  ## little speed gain, or slower
             temp <- sapply(1:data$nc, onehistory, pmix = pmix, gk = gk, USE.NAMES = FALSE)
         }
     }
-    else {
-        if (data$multi) {
-            clusterExport(cluster, c("pmix", "hk", "haztemp"), environment())
-            temp <- parSapply(cluster, 1:data$nc, onehistorymulti, pmix = pmix, hk = hk,
-                              USE.NAMES = FALSE )
-        }
-        else {
-            clusterExport(cluster, c("pmix", "gk"), environment())
-            temp <- parSapply(cluster, 1:data$nc, onehistory, pmix = pmix, gk = gk,
-                              USE.NAMES = FALSE )
-        }
-    }
     comp[1] <- sum(temp)
     #####################################################################
     # Component 2: Probability of unobserved histories
-    
-    if (type %in% c(7:14,24,25, 37:44)) {
+
+    if (type %in% c(7:14,24,25, 30, 31)) {
         if (data$learnedresponse) {   ## use model for naive animal
-            temp <- makegkcpp(
-                as.integer(nrow(realparval0)), # number of rows in lookup table
-                as.integer(data$k),            # detectors
-                as.integer(data$m),            # mask points
-                as.integer(data$detectfn),
-                as.integer(.openCRstuff$sigmai[type]),   # column index of sigma in realparval
-                as.double(realparval0),
-                as.double(unlist(trps)),
-                as.double(unlist(data$mask)))
-            gk <- temp[[1]]
-            hk <- temp[[2]]
+            gk <- makegkParallelcpp (as.integer(data$detectfn),
+                                       as.integer(.openCRstuff$sigmai[type]),
+                                       as.integer(data$details$grain),
+                                       as.matrix(realparval0),
+                                       as.matrix(trps),
+                                       as.matrix(data$mask))[[1]]
         }
+        ## else use gk as gk0
+
         pdot <- rep(0, data$nc)
         for (x in 1:data$details$nmix) {   # loop over latent classes
-            
             if (data$details$R) {
                 pch1 <-  PCH1secr(
-                    type, 
-                    x, 
-                    1,            # data$nc, 
+                    type,
+                    as.logical(data$design0$individual),
+                    x,
+                    data$nc,
                     data$J,
                     data$cumss,
-                    data$k, 
-                    data$m, 
+                    data$k,
+                    data$m,
                     realparval0,
-                    PIA0[1,,,,drop = FALSE], 
-                    PIAJ[1,,,drop = FALSE], 
-                    gk, 
-                    binomN, 
+                    PIA0,
+                    PIAJ,
+                    gk,
+                    binomN,
                     data$usge,
-                    data$intervals, 
-                    data$moveargsi, 
+                    data$intervals,
+                    data$moveargsi,
                     data$movemodel,
-                    data$kernel, 
-                    data$mqarray, 
-                    data$cellsize) 
+                    data$kernel,
+                    data$mqarray,
+                    data$cellsize)
             }
             else {
-                pch1 <-  PCH1secrcpp(
-                    as.integer(type),
-                    as.integer(x-1),
-                    as.integer(1),
-                    as.integer(data$J),
-                    as.integer(data$cumss),
-                    as.integer(data$k),
-                    as.integer(data$m),
-                    as.double (unlist(realparval0)),
-                    as.integer(nrow(realparval0)),
-                    as.integer(PIA0[1,,,]),
-                    as.integer(PIAJ[1,,]),
-                    as.double (gk),             ## gk0
-                    as.integer(binomN),
-                    as.double (data$usge),
-                    as.double (data$intervals),
-                    as.integer(data$moveargsi),
-                    as.integer(data$movemodel),
-                    as.character(data$usermodel),
-                    as.integer(nrow(data$kernel)),
-                    as.integer(as.matrix(data$kernel)),
-                    as.integer(data$mqarray),
-                    as.double (data$cellsize))
+                if (data$ncores==1) {
+                    pch1 <-  PCH1secrcpp(
+                        as.integer(type),
+                        as.logical(data$design0$individual),
+                        as.integer(x-1),
+                        as.integer(data$nc),
+                        as.integer(data$J),
+                        as.integer(data$cumss),
+                        as.integer(data$k),
+                        as.integer(data$m),
+                        as.matrix (realparval0),
+                        as.integer(PIA0),
+                        as.integer(PIAJ),
+                        as.double (gk),
+                        as.integer(binomN),
+                        as.matrix (data$usge),
+                        as.double (data$intervals),
+                        as.integer(data$moveargsi),
+                        as.integer(data$movemodel),
+                        as.character(data$usermodel),
+                        as.matrix(data$kernel),
+                        as.matrix(data$mqarray),
+                        as.double (data$cellsize))
+                }
+                else {
+                    pch1 <-  PCH1secrparallelcpp(
+                        as.integer(x-1),
+                        as.integer(type),
+                        as.integer(data$details$grain),
+                        as.logical(data$design0$individual),
+                        as.integer(data$J),
+                        as.integer(data$m),
+                        as.integer(data$nc),
+                        as.integer(data$cumss),
+                        as.matrix (realparval0),
+                        as.integer(PIA0),
+                        as.integer(PIAJ),
+                        as.double (gk),
+                        as.integer(binomN),
+                        as.matrix (data$usge),
+                        as.double (data$intervals),
+                        as.integer(data$moveargsi),
+                        as.integer(data$movemodel),
+                        as.matrix(data$kernel),
+                        as.matrix(data$mqarray),
+                        as.double (data$cellsize))
+                }
             }
-            pch1 <- rep(pch1, data$nc)      ## let's ignore individual variation!!!
             pdot <- pdot + pmix[x] * pch1
         }
         comp[2] <- - sum(freq * log(pdot))
     }
     #####################################################################
     # Component 3: Probability of observing nc animals
-    
-    if (type %in% c(7,8,12,13,14,24)) {
-        if (type %in% c(7, 12, 13, 24))
+
+    if (type %in% c(7,8,12,13,14,24, 31)) {
+        if (type %in% c(7, 12, 13, 24, 31))
             Dsuper <- realparval[nrow(realparval)*3+1] # Dsuper direct
         else  {        # type %in% c(8, 14))
             Dsuper <- getD(type, data$J, data$details$nmix, pmix,
@@ -661,16 +698,16 @@ open.secr.loglikfn <- function (beta, dig = 3, betaw = 8, cluster = NULL, oneeva
         N <- Dsuper * A
         # impose constraint: return with invalid result code if not possible
         if (N < ncf) return;
-        
+
         meanpdot <- data$nc / sum(1/pdot)
         ## cf CLmeanesa in 'secr'
-        
+
         comp[3] <- switch (data$distrib+1,
                            dpois(ncf, N * meanpdot, log = TRUE),
                            lnbinomial (ncf, N, meanpdot),
                            NA)
     }
-    
+    #####################################################################
 
     ## optional multinomial term (not if CJS)
     if (data$details$multinom & !(type %in% c(6,36))) {
@@ -685,16 +722,18 @@ open.secr.loglikfn <- function (beta, dig = 3, betaw = 8, cluster = NULL, oneeva
         cat("Total ", format(loglik, digits = 10), "\n")
         browser()
     }
-
+    ## optionally display message on console for this iteration
     .openCRstuff$iter <- .openCRstuff$iter + 1
     if (data$details$trace) {
-        if (!is.null(data$details$fixedbeta))
-            beta <- beta[is.na(data$details$fixedbeta)]
-        cat(format(.openCRstuff$iter, width=4),
-            formatC(round(loglik,dig), format='f', digits=dig, width=betaw),
-            formatC(beta, format='f', digits=dig+1, width=betaw),
-            '\n', sep = " ")
-        flush.console()
+        if ((.openCRstuff$iter %% data$details$trace) == 0) {
+            if (!is.null(data$details$fixedbeta))
+                beta <- beta[is.na(data$details$fixedbeta)]
+            message(format(.openCRstuff$iter, width=4), "   ",
+                    formatC(round(loglik,dig), format='f', digits=dig, width=betaw+2),
+                    formatC(beta, format='f', digits=dig+1, width=betaw+1),
+                    sep = " ")
+            flush.console()
+        }
     }
 
     if (oneeval) {
